import { Hash } from '@loaders.gl/crypto';
import * as crypto from 'crypto';
export class NodeHash extends Hash {
  constructor(options) {
    var _this$options, _this$options$crypto;
    super();
    this.name = 'crypto-node';
    this.options = void 0;
    this._algorithm = void 0;
    this._hash = void 0;
    this.options = options;
    if (!((_this$options = this.options) !== null && _this$options !== void 0 && (_this$options$crypto = _this$options.crypto) !== null && _this$options$crypto !== void 0 && _this$options$crypto.algorithm)) {
      throw new Error(this.name);
    }
  }
  async hash(input, encoding) {
    var _this$options2, _this$options2$crypto, _this$options2$crypto2;
    const algorithm = (_this$options2 = this.options) === null || _this$options2 === void 0 ? void 0 : (_this$options2$crypto = _this$options2.crypto) === null || _this$options2$crypto === void 0 ? void 0 : (_this$options2$crypto2 = _this$options2$crypto.algorithm) === null || _this$options2$crypto2 === void 0 ? void 0 : _this$options2$crypto2.toLowerCase();
    try {
      var _crypto$createHash;
      if (!crypto.createHash) {
        throw new Error('crypto.createHash not available');
      }
      const hash = (_crypto$createHash = crypto.createHash) === null || _crypto$createHash === void 0 ? void 0 : _crypto$createHash.call(crypto, algorithm);
      const inputArray = new Uint8Array(input);
      return hash.update(inputArray).digest('base64');
    } catch (error) {
      throw Error(`${algorithm} hash not available. ${error}`);
    }
  }
  hashBatches(asyncIterator) {
    try {
      var _this = this;
      let encoding = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 'base64';
      return async function* () {
        var _crypto$createHash2, _this$options3, _this$options3$crypto, _this$options3$crypto2, _this$options4, _this$options4$crypto, _this$options4$crypto2;
        if (!crypto.createHash) {
          throw new Error('crypto.createHash not available');
        }
        const hash = (_crypto$createHash2 = crypto.createHash) === null || _crypto$createHash2 === void 0 ? void 0 : _crypto$createHash2.call(crypto, (_this$options3 = _this.options) === null || _this$options3 === void 0 ? void 0 : (_this$options3$crypto = _this$options3.crypto) === null || _this$options3$crypto === void 0 ? void 0 : (_this$options3$crypto2 = _this$options3$crypto.algorithm) === null || _this$options3$crypto2 === void 0 ? void 0 : _this$options3$crypto2.toLowerCase());
        for await (const chunk of asyncIterator) {
          const inputArray = new Uint8Array(chunk);
          hash.update(inputArray);
          yield chunk;
        }
        const digest = hash.digest(encoding);
        (_this$options4 = _this.options) === null || _this$options4 === void 0 ? void 0 : (_this$options4$crypto = _this$options4.crypto) === null || _this$options4$crypto === void 0 ? void 0 : (_this$options4$crypto2 = _this$options4$crypto.onEnd) === null || _this$options4$crypto2 === void 0 ? void 0 : _this$options4$crypto2.call(_this$options4$crypto, {
          hash: digest
        });
      }();
    } catch (e) {
      return Promise.reject(e);
    }
  }
}
//# sourceMappingURL=node-hash.js.map