import fsPromise from 'fs/promises';
import { NodeFile } from "./node-file.js";
import { fetchNode } from "./fetch-node.js";
export class NodeFileSystem {
  constructor() {
    this.readable = true;
    this.writable = true;
  }
  async readdir() {
    let dirname = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : '.';
    let options = arguments.length > 1 ? arguments[1] : undefined;
    return await fsPromise.readdir(dirname, options);
  }
  async stat(path) {
    const info = await fsPromise.stat(path, {
      bigint: true
    });
    return {
      size: Number(info.size),
      bigsize: info.size,
      isDirectory: info.isDirectory()
    };
  }
  async unlink(path) {
    return await fsPromise.unlink(path);
  }
  async fetch(path, options) {
    return await fetchNode(path, options);
  }
  async openReadableFile(path) {
    let flags = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 'r';
    return new NodeFile(path, flags);
  }
  async openWritableFile(path) {
    let flags = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 'w';
    let mode = arguments.length > 2 ? arguments[2] : undefined;
    return new NodeFile(path, flags, mode);
  }
}
//# sourceMappingURL=node-filesystem.js.map